﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using CMS.SettingsProvider;
using CMS.GlobalHelper;


/// <summary>
/// Summary description for CustomDataContainer
/// </summary>
public class CustomDataContainer : IDataContainer
{
    #region "Variables"

    private List<string> mColumnNames = null;
    private string mMarketPriceSourceName = null;
    private double mMarketPriceValue = Double.MinValue;

    #endregion


    #region "Properties"
    
    /// <summary>
    /// Gets or sets the site owner
    /// </summary>
    public string MarketPriceSourceName
    {
        get
        {
            return mMarketPriceSourceName;
        }
        set
        {
            mMarketPriceSourceName = value;
        }
    }


    /// <summary>
    /// Gets or sets the date until which the site is valid
    /// </summary>
    public double MarketPriceValue
    {
        get
        {
            return mMarketPriceValue;
        }
        set
        {
            mMarketPriceValue = value;
        }
    }

    #endregion


    public CustomDataContainer()
    {
        //
        // TODO: Add constructor logic here
        //
    }


    #region "IDataContainer members"
    
    /// <summary>
    /// Gets the column names
    /// </summary>
    public List<string> ColumnNames
    {
        get
        {
            if (mColumnNames == null)
            {
                mColumnNames = new List<string>();
                mColumnNames.Add("MarketPriceSourceName");
                mColumnNames.Add("MarketPriceValue");
            }

            return mColumnNames;
        }
    }

    
    /// <summary>
    /// Returns column values
    /// </summary>
    /// <param name="columnName">Column value to return</param>
    public object this[string columnName]
    {
        get
        {
            return this.GetValue(columnName);
        }
        set
        {
            this.SetValue(columnName, value);
        }
    }


    /// <summary>
    /// Returns true if columns exists.
    /// </summary>
    public bool ContainsColumn(string columnName)
    {
        switch (columnName.ToLower())
        {
            case "marketpricesourcename":
            case "marketpricevalue":
                return true;

            default:
                return false;
        }
    }


    /// <summary>
    /// Gets the object value
    /// </summary>
    /// <param name="columnName">Column name</param>
    public object GetValue(string columnName)
    {
        switch (columnName.ToLower())
        {
            case "marketpricesourcename":
                return MarketPriceSourceName;

            case "marketpricevalue":
                return MarketPriceValue;

            default:
                return null;
        }
    }


    /// <summary>
    /// Sets the field value
    /// </summary>
    /// <param name="columnName">Column name</param>
    /// <param name="value">New value</param>
    public bool SetValue(string columnName, object value)
    {
        switch (columnName.ToLower())
        {
            case "marketpricesourcename":
                MarketPriceSourceName = ValidationHelper.GetString(value, "");
                return true;

            case "marketpricevalue":
                MarketPriceValue = ValidationHelper.GetDouble(value, Double.MinValue);
                return true;

            default:
                return false;
        }
    }


    /// <summary>
    /// Tries to get the value from the object
    /// </summary>
    /// <param name="columnName">Column name</param>
    /// <param name="value">Returns the value</param>
    public bool TryGetValue(string columnName, out object value)
    {
        switch (columnName.ToLower())
        {
            case "marketpricesourcename":
                value = MarketPriceSourceName;
                return true;

            case "marketpricevalue":
                value = MarketPriceValue;
                return true;

            default:
                value = null;
                return false;
        }
    }

    #endregion


    #region "Methods"

    /// <summary>
    /// Returns true if the market info are valid
    /// </summary>
    public bool IsValid()
    {
        return (!string.IsNullOrEmpty(MarketPriceSourceName) && (MarketPriceValue > Double.MinValue));
    }

    #endregion
}