﻿using System;
using System.Web;
using System.Data;
using System.Collections.Generic;

using CMS.Ecommerce;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.OnlineMarketing;

/// <summary>
/// Sample order info provider. 
/// Can be registered either by replacing the OrderInfoProvider.ProviderObject (uncomment the line in SampleECommerceModule.cs) or through cms.extensibility section of the web.config
/// </summary>
public class CustomOrderInfoProvider : OrderInfoProvider
{
    #region "Example: Add specific amount of credit to each customer after he makes an order."

    ///// <summary>
    ///// Sets (updates or inserts) specified order.
    ///// </summary>
    ///// <param name="orderObj">Order to be set</param>  
    //protected override void SetOrderInfoInternal(OrderInfo orderObj)
    //{
    //    // Remember whether it is a new order
    //    bool newOrder = ((orderObj != null) && (orderObj.OrderID <= 0));

    //    // Add/Update the order
    //    base.SetOrderInfoInternal(orderObj);

    //    // Add credit to the customer for a new order with the total price higher than 1000 
    //    if (newOrder && (orderObj.OrderTotalPriceInMainCurrency > 1000))
    //    {
    //        // Initialize credit event
    //        CreditEventInfo extraCredit = new CreditEventInfo();
    //        extraCredit.EventName = string.Format("Credit for your order: {0}", orderObj.OrderID);
    //        extraCredit.EventDate = DateTime.Now;
    //        extraCredit.EventDescription = "Thank you for your order.";
    //        extraCredit.EventCustomerID = orderObj.OrderCustomerID;

    //        // Credit in site main currency which will be added to the customer
    //        extraCredit.EventCreditChange = 10;

    //        // Set credit as site credit or as global credit according to the settings
    //        extraCredit.EventSiteID = ECommerceHelper.GetSiteID(CMSContext.CurrentSiteID, ECommerceSettings.USE_GLOBAL_CREDIT);

    //        // Save credit event
    //        CreditEventInfoProvider.SetCreditEventInfo(extraCredit);
    //    }
    //}

    #endregion


    #region "Webinar samples"

    protected override void ProcessMembershipInternal(OrderInfo oi, OrderItemInfo oii, SKUInfo skui, UserInfo ui, DateTime now)
    {
        if ((oi != null) && (oii != null) && (ui != null) && (skui != null) && (skui.SKUProductType == SKUProductTypeEnum.Membership))
        {
            bool isPaid = oi.OrderIsPaid;

            // Get membership information
            CMS.SiteProvider.MembershipInfo mi = CMS.SiteProvider.MembershipInfoProvider.GetMembershipInfo(skui.SKUMembershipGUID, skui.SKUSiteID);
            if (mi != null)
            {
                // Get all users from the system that belongs to the same OM account as user paying for the order
                InfoDataSet<UserInfo> accountUsers = GetAllAccountUsers(ui.UserID);
                if (!DataHelper.DataSourceIsEmpty(accountUsers))
                {
                    // Process membership for each user separately
                    foreach (UserInfo aui in accountUsers)
                    {
                        if (isPaid)
                        {
                            // Get membership user information
                            MembershipUserInfo mui = MembershipUserInfoProvider.GetMembershipUserInfo(mi.MembershipID, aui.UserID);
                            if (mui == null)
                            {
                                // Create new membership for the user
                                mui = new MembershipUserInfo();

                                mui.UserID = aui.UserID;
                                mui.MembershipID = mi.MembershipID;
                                mui.ValidTo = now;
                            }

                            // If user's membership validity is not unlimited
                            if (mui.ValidTo.CompareTo(DateTimeHelper.ZERO_TIME) != 0)
                            {
                                // If user's memberhsip already expired
                                if (mui.ValidTo.CompareTo(now) < 0)
                                {
                                    mui.ValidTo = now;
                                }

                                DateTime newValidTo = DateTimeHelper.ZERO_TIME;

                                // Prolong user's membership accordingly
                                if (skui.SKUValidity == ValidityEnum.Until)
                                {
                                    newValidTo = skui.SKUValidUntil;
                                }
                                else
                                {
                                    newValidTo = DateTimeHelper.GetValidTo(mui.ValidTo, skui.SKUValidity, skui.SKUValidFor * oii.OrderItemUnitCount);
                                }

                                // If new validity is unlimited or is later than original
                                if ((newValidTo.CompareTo(DateTimeHelper.ZERO_TIME) == 0) || (newValidTo.CompareTo(mui.ValidTo) > 0))
                                {
                                    // Set valid to date
                                    mui.ValidTo = newValidTo;

                                    // Set send notification flag
                                    mui.SendNotification = true;

                                    // Save membership user information
                                    MembershipUserInfoProvider.SetMembershipUserInfo(mui);
                                }
                            }
                        }
                        else
                        {
                            // If order is not paid, let the default implementation rollback user's membership prolongation
                            base.ProcessMembershipInternal(oi, oii, skui, aui, now);
                        }
                    }
                }
            }
        }
    }


    /// <summary>
    /// Returns collection of users that belong to the same OM account as specified user.
    /// </summary>
    /// <param name="userId">ID of the user to use.</param>
    private InfoDataSet<UserInfo> GetAllAccountUsers(int userId)
    {
        InfoDataSet<UserInfo> result = null;

        if (userId > 0)
        {
            int memberType = (int)MemberTypeEnum.CmsUser;

            // Try to get any relationship between current contact and e-commerce customer (there can be more than just one customer)
            string where = "(RelatedID = " + userId + ")";
            where = SqlHelperClass.AddWhereCondition(where, "MemberType =" + memberType);

            // Try to get customers associated with current contact
            InfoDataSet<CMS.OnlineMarketing.MembershipInfo> contactUsers = CMS.OnlineMarketing.MembershipInfoProvider.GetRelationships(where, null, 1, "ActiveContactID");
            if (!DataHelper.DataSourceIsEmpty(contactUsers)) 
            {
                CMS.OnlineMarketing.MembershipInfo mi = contactUsers.Items[0];

                CMS.OnlineMarketing.ContactInfo ci = CMS.OnlineMarketing.ContactInfoProvider.GetContactInfo(mi.ActiveContactID);
                if (ci != null) 
                {
                    int accountId = ci.Accounts[0].GetIntegerValue("AccountID", 0);
                    if (accountId > 0) 
                    {
                        InfoDataSet<AccountContactInfo> accountContacts = AccountContactInfoProvider.GetRelationships("AccountID = " + accountId, null, -1, "ContactID");
                        if (!DataHelper.DataSourceIsEmpty(accountContacts))
                        {
                            string contactIds = "";

                            foreach (AccountContactInfo aci in accountContacts) 
                            {
                                if (contactIds != "") 
                                {
                                    contactIds += ", ";
                                }

                                contactIds += aci.ContactID;
                            }

                            if (contactIds != "")
                            {
                                string userWhere = "ActiveContactID IN (" + contactIds + ")";
                                userWhere = SqlHelperClass.AddWhereCondition(userWhere, "ActiveContactID = OriginalContactID");
                                userWhere = SqlHelperClass.AddWhereCondition(userWhere, "MemberType =" + memberType);

                                InfoDataSet<CMS.OnlineMarketing.MembershipInfo> accountUsers = CMS.OnlineMarketing.MembershipInfoProvider.GetRelationships(userWhere, null, -1, "RelatedID");
                                if (!DataHelper.DataSourceIsEmpty(accountUsers))
                                {
                                    string userIds = "";

                                    foreach (CMS.OnlineMarketing.MembershipInfo mui in accountUsers)
                                    {
                                        if (userIds != "")
                                        {
                                            userIds += ", ";
                                        }

                                        userIds += mui.RelatedID;
                                    }

                                    if (userIds != "") 
                                    {
                                        userWhere = "UserID IN (" + userIds + ")";

                                        result = UserInfoProvider.GetUsers(userWhere, null, -1, "UserID");
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return result;
    }

    #endregion
}