﻿using System;
using System.Web;
using System.Collections.Generic;

using CMS.Ecommerce;
using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.OnlineMarketing;

using MembershipInfo = CMS.OnlineMarketing.MembershipInfo;
using MembershipInfoProvider = CMS.OnlineMarketing.MembershipInfoProvider;
using System.Data; 


/// <summary>
/// Sample shopping cart info provider. 
/// Can be registered either by replacing the SKUInfoProvider.ProviderObject (uncomment the line in SampleECommerceModule.cs) or through cms.extensibility section of the web.config
/// </summary>
public class CustomSKUInfoProvider : SKUInfoProvider
{
    #region "Example: Custom SKU base price: 1) SKU price according to the culture 2) SKU price according to the SKU status"

    ///// <summary>
    ///// Returns catalogue price of the given product based on the SKU data and shopping cart data. 
    ///// By default price is loaded from the SKUPrice field.
    ///// </summary>
    ///// <param name="sku">SKU dta</param>
    ///// <param name="cart">Shopping cart data</param>
    //protected override double GetSKUPriceInternal(SKUInfo sku, ShoppingCartInfo cart)
    //{
    //    double price = 0;

    //    // 1) Get base SKU price according to the shopping cart data (current culture)
    //    if (cart != null)
    //    {
    //        switch (cart.ShoppingCartCulture.ToLowerCSafe())
    //        {
    //            case "en-us":
    //                // Get price form SKUEnUsPrice field
    //                price = ValidationHelper.GetDouble(sku.GetValue("SKUEnUsPrice"), 0);
    //                break;

    //            case "cs-cz":
    //                // Get price form SKUCsCzPrice field
    //                price = ValidationHelper.GetDouble(sku.GetValue("SKUCsCzPrice"), 0);
    //                break;
    //        }
    //    }

    //    //// 2) Get base SKU price according to the product properties (product status)
    //    //PublicStatusInfo status = PublicStatusInfoProvider.GetPublicStatusInfo(sku.SKUPublicStatusID);
    //    //if ((status != null) && (status.PublicStatusName.ToLowerCSafe() == "discounted"))
    //    //{
    //    //    // Get price form SKUDiscountedPrice field
    //    //    price = ValidationHelper.GetDouble(sku.GetValue("SKUDiscountedPrice"), 0);
    //    //}

    //    if (price == 0)
    //    {
    //        // Get price from the SKUPrice field by default
    //        return base.GetSKUPriceInternal(sku, cart);
    //    }
    //    else
    //    {
    //        // Return custom price
    //        return price;
    //    }
    //}

    #endregion


    #region "Example: Custom SKU discount for the users in a specific role"

    ///// <summary>
    ///// Returns list of catalogue discounts which should be applied to the given product based on the product data and shopping cart data.
    ///// </summary>
    ///// <param name="sku">SKU data</param>
    ///// <param name="cart">Shopping cart data</param>
    ///// <param name="forCart">True - This method is called within the shopping cart calculations, False - This method is called within the displaying price on the live site.</param>
    //protected override List<IItemDiscount> GetSKUDiscountsInternal(SKUInfo sku, ShoppingCartInfo cart, bool forCart)
    //{
    //    // SKU number of product A
    //    string skuA = "A";

    //    // Get SKU default discounts
    //    List<IItemDiscount> discounts = base.GetSKUDiscountsInternal(sku, cart, forCart);

    //    // Add extra discount to product A 
    //    if (sku.SKUNumber.ToLowerCSafe() == skuA.ToLowerCSafe())
    //    {
    //        // Get user information
    //        UserInfo user = cart.User;

    //        // Add discount if user is in VIP role
    //        if ((user != null) && user.IsInRole("VIP", cart.SiteName))
    //        {                
    //            AddDiscountForVIPRole(sku, discounts);
    //        }
    //    }

    //    return discounts;
    //}


    ///// <summary>
    ///// Adds 10% discount to the given sku.
    ///// </summary>
    ///// <param name="sku">SKU data</param>
    ///// <param name="discounts">Discounts of the sku</param>
    //private void AddDiscountForVIPRole(SKUInfo sku, List<IItemDiscount> discounts)
    //{
    //    // Create 10% discount
    //    ItemDiscount discount = new ItemDiscount()
    //    {
    //        ItemDiscountID = "DISCOUNT_VIP",
    //        ItemDiscountDisplayName = "Discount for VIP customers",
    //        ItemDiscountValue = 10,                
    //    };

    //    // Add custom discount to discounts which will be applied to the product
    //    discounts.Add(discount);        
    //}

    #endregion


    #region "Webinar samples"

    /// <summary>
    /// Returns catalogue price of the given product based on the SKU data and shopping cart data. 
    /// By default price is loaded from the SKUPrice field.
    /// </summary>
    /// <param name="sku">SKU dta</param>
    /// <param name="cart">Shopping cart data</param>
    protected override double  GetSKUPriceInternal(SKUInfo sku, ShoppingCartInfo cart, string column)
    {
        double result = 0.0;

        if (cart != null)
        {
            CustomerInfo currentCustomer = null;
            InfoDataSet<MembershipInfo> contactCustomers = null;

            // Try to get existing customer or check whether the current contact has any customers reltionships
            if (!TryGetExistingCustomers(ref currentCustomer, ref contactCustomers))
            {
                // If no customer exists for the current contact display special offer price
                result = ValidationHelper.GetDouble(sku.GetValue("SKUNewCustomerPrice"), 0);
            }
        }

        // Get default price if nothing applied above
        if (result == 0.0)
        {            
            result = base.GetSKUPriceInternal(sku, cart, column);
        }
        
        return result;
    }


    /// <summary>
    /// Returns list of catalogue discounts which should be applied to the given product based on the product data and shopping cart data.
    /// </summary>
    /// <param name="sku">SKU data</param>
    /// <param name="cart">Shopping cart data</param>
    /// <param name="forCart">True - This method is called within the shopping cart calculations, False - This method is called within the displaying price on the live site.</param>
    protected override List<IItemDiscount> GetSKUDiscountsInternal(SKUInfo sku, ShoppingCartInfo cart, bool forCart)
    {
        // Get SKU default discounts
        List<IItemDiscount> result = base.GetSKUDiscountsInternal(sku, cart, forCart);

        int productId = sku.SKUID;
        CustomerInfo customer = null;
        InfoDataSet<MembershipInfo> contactCustomers = null;

        // Add extra discount to product A 
        if (TryGetExistingCustomers(ref customer, ref contactCustomers))
        {
            InfoDataSet<OrderInfo> oi = null;

            if (customer != null)
            {
                // Check orders for customer
                oi = GetExistingOrders(productId, customer, null);
            }
            else if (!DataHelper.DataSourceIsEmpty(contactCustomers)) 
            {
                // Check orders for contact customers
                oi = GetExistingOrders(productId, null, contactCustomers);
            }

            // If product purchased previously
            if (!DataHelper.DataSourceIsEmpty(oi))
            {
                // Create 10% discount
                ItemDiscount newCustomerDiscount = new ItemDiscount()
                {
                    ItemDiscountID = "RepeatedPurchaseDiscount10%",
                    ItemDiscountDisplayName = "Discount 10% for repeated purchase",
                    ItemDiscountValue = 10,
                    // ItemDiscountIsFlat = true,
                };

                // Add custom discount to discounts which will be applied to the product
                result.Add(newCustomerDiscount);
            }
        }

        return result;
    }


    /// <summary>
    /// 
    /// </summary>
    /// <param name="productId"></param>
    /// <param name="customer"></param>
    /// <param name="contactCustomers"></param>
    private static InfoDataSet<OrderInfo> GetExistingOrders(int productId, CustomerInfo customer, InfoDataSet<MembershipInfo> contactCustomers)
    {
        InfoDataSet<OrderInfo> result = null;

        // Existing customer
        if (customer != null)
        {
            result = GetCustomerOrders(productId, customer.CustomerID);
        }
        else if (!DataHelper.DataSourceIsEmpty(contactCustomers)) 
        {
            // Go through all contact customers
            foreach (MembershipInfo customerRelationship in contactCustomers) 
            {
                int customerId = customerRelationship.RelatedID;

                // Try to find qualifying order for current contact customer
                result = GetCustomerOrders(productId, customerId);
                if (!DataHelper.DataSourceIsEmpty(result)) 
                {
                    // As soon as there is at least one order stop searching
                    break;
                }
            }
        }

        return result;
    }


    /// <summary>
    /// 
    /// </summary>
    /// <param name="productId"></param>
    /// <param name="customerId"></param>
    private static InfoDataSet<OrderInfo> GetCustomerOrders(int productId, int customerId)
    {
        InfoDataSet<OrderInfo> result = null;

        if ((productId > 0) && (customerId > 0))
        {
            string where = "OrderCustomerID = " + customerId;
            where = SqlHelperClass.AddWhereCondition(where, "OrderID IN (SELECT OrderItemOrderID FROM COM_OrderItem WHERE (OrderItemSKUID = " + productId + "))");
            where = SqlHelperClass.AddWhereCondition(where, "DATEDIFF(dd, GETDATE(), OrderDate) <= 30");

            // Check if given product was already purchased by customer
            result = OrderInfoProvider.GetOrders(where, null, 1, "OrderID");
        }

        return result;
    }


    /// <summary>
    /// Checks if current user is existing customer. 
    /// </summary>
    /// <param name="contactCustomers">Collection of contact - customer relationships.</param>
    /// <param name="singleCustomer">Existing e-commerce customer.</param>
    private bool TryGetExistingCustomers(ref CustomerInfo singleCustomer, ref InfoDataSet<MembershipInfo> contactCustomers)
    {
        singleCustomer = ECommerceContext.CurrentCustomer;

        // If current user is not existing customer proceed
        if (singleCustomer == null)
        {
            // Get current contact information
            ContactInfo ci = OnlineMarketingContext.CurrentContact;
            if (ci != null)
            {
                int contactId = ci.ContactID;
                int memberType = (int)MemberTypeEnum.EcommerceCustomer;

                // Try to get any relationship between current contact and e-commerce customer (there can be more than just one customer)
                string where = "(OriginalContactID = " + contactId + ") OR (ActiveContactID = " + contactId + ")";
                where = SqlHelperClass.AddWhereCondition(where, "MemberType =" + memberType);

                // Try to get customers associated with current contact
                contactCustomers = MembershipInfoProvider.GetRelationships(where, null, 1, "RelatedID");
            }
        }

        // Return TRUE if there is existing customer or customer relationship was found for current contact
        return ((singleCustomer != null) || !DataHelper.DataSourceIsEmpty(contactCustomers));
    }

    #endregion
}