﻿using System;
using System.Web;

using CMS.Ecommerce;
using CMS.SettingsProvider;
using CMS.SiteProvider;

/// <summary>
/// Sample shipping option info provider. 
/// Can be registered either by replacing the ShippingOptionInfoProvider.ProviderObject (uncomment the line in SampleECommerceModule.cs) or through cms.extensibility section of the web.config
/// </summary>
public class CustomShippingOptionInfoProvider : ShippingOptionInfoProvider
{
    #region "Example: Custom shipping calcalation"

    ///// <summary>
    ///// Calculates shipping charge for the given shopping cart.
    ///// Shipping taxes are not included. Result is in site main currency.
    ///// </summary>
    ///// <param name="cartObj">Shopping cart data</param>
    //protected override double CalculateShippingInternal(ShoppingCartInfo cart)
    //{
    //    // ------------------------
    //    // Please note:         
    //    // You can customize this method to calculate shipping by a on-line shipping calculation service as well.
    //    // All the data which might be required for the calculation service is stored in the ShoppingCartInfo object, e.g.:
    //    // - use AddressInfoProvider.GetAddresInfo(cart.ShoppingCartBillingAddressID) to get billing address info
    //    // - use AddressInfoProvider.GetAddresInfo(cart.ShoppingCartShippingAddressID) to get shipping address info        
    //    // etc.
    //    // ------------------------

    //    // Calculates shipping based on customer's billing address country
    //    if (cart != null)
    //    {
    //        // Get site name
    //        string siteName = cart.SiteName;

    //        if ((cart.User != null) && (cart.User.IsInRole("VIP", siteName)))
    //        {
    //            // Free shipping for VIP customers
    //            return 0;
    //        }
    //        else
    //        {
    //            // Get shipping address details
    //            AddressInfo address = AddressInfoProvider.GetAddressInfo(cart.ShoppingCartShippingAddressID);
    //            if (address != null)
    //            {
    //                // Get shipping address country
    //                CountryInfo country = CountryInfoProvider.GetCountryInfo(address.AddressCountryID);
    //                if ((country != null) && (country.CountryName.ToLowerCSafe() != "usa"))
    //                {
    //                    // Get extra shipping for non-usa customers from 'ShippingExtraCharge' custom setting 
    //                    double extraCharge = SettingsKeyProvider.GetDoubleValue("ShippingExtraCharge");

    //                    // Add an extra charge to standard shipping price for non-usa customers
    //                    return base.CalculateShippingInternal(cart) + extraCharge;
    //                }
    //            }
    //        }
    //    }

    //    // Calculate shipping option without tax in default way
    //    return base.CalculateShippingInternal(cart);
    //}

    #endregion


    #region "Webinar samples"

    /// <summary>
    /// Calculates shipping charge for the given shopping cart.
    /// Shipping taxes are not included. Result is in site main currency.
    /// </summary>
    /// <param name="cartObj">Shopping cart data</param>
    protected override double CalculateShippingInternal(ShoppingCartInfo cart)
    {
        // Calculate shipping option without tax in default way
        double result = 0;

        // Calculates shipping for products in the cart if shipping option selected is what we want to customize
        if ((cart != null) && (cart.ShoppingCartShippingOptionID > 0) && (cart.ShippingOption.ShippingOptionName.ToLower() == "ecommercesite.2shipping"))
        {
            // Get site name
            string siteName = cart.SiteName;

            // Go thhrough items in the cart and get product specific shipping costs
            foreach (ShoppingCartItemInfo cartItem in cart.CartItems)
            {
                // Proceed only if the item is not product option and shipping is required
                if (cartItem.SKU.SKUNeedsShipping && !cartItem.IsProductOption)
                {
                    // Custom SKU property specifying minimal number of units of product qualifying for free shipping
                    int freeShippingUnits = cartItem.SKU.GetIntegerValue("FreeShippingWhenNoUnits", 0);

                    // Calculate shipping costs if product does not qualify for free shipping
                    if ((freeShippingUnits == 0) || (cartItem.CartItemUnits < freeShippingUnits))
                    {
                        double itemShipping = GetSKUShippingPrice(cartItem, cart);

                        // Add to the custom data to be displayed in shipping summary on live site
                        cart.ShoppingCartCustomData["CartItemID_" + cartItem.CartItemID.ToString()] = itemShipping;

                        // Add item shipping to total shipping price
                        result += itemShipping;
                    }
                }
            }
        }
        else 
        {
            // If the shipping option not recognized let default implementation take care of shipping costs
            result = base.CalculateShippingInternal(cart);
        }
                
        return result;
    }


    /// <summary>
    /// Returns SKU shipping price based on item weight and dimensions.
    /// </summary>
    /// <param name="cartItem">Item to calculate shipping for.</param>
    private double GetSKUShippingPrice(ShoppingCartItemInfo cartItem, ShoppingCartInfo cart)
    {
        double result = 0;

        if (cartItem != null) 
        {
            // Get info on cart item
            double weight = cartItem.UnitWeight;
            double height = cartItem.SKU.SKUHeight;
            double width = cartItem.SKU.SKUWidth;
            double depth = cartItem.SKU.SKUDepth;

            // This part can be implemented either manually OR you can submitt data to the 3rd party shipping provider to retrieve actual pricing
            
            // USPS Priority Mail Boxes flat-rate weight limit
            double flatRateWeightLimit = 70.0;
            if (weight <= flatRateWeightLimit)
            {
                // Small PMB dimensions
                double sPMBWidth = 8;
                double sPMBHeight = 5;
                double sPMBDepth = 2;

                // Medium PMB dimensions
                double mPMBWidth = 13;
                double mPMBHeight = 11;
                double mPMBDepth = 4;

                // Large PMB dimensions
                double lPMBWidth = 23;
                double lPMBHeight = 11;
                double lPMBDepth = 3;

                // Small package
                if ((height <= sPMBHeight) && (width <= sPMBWidth) && (depth <= sPMBDepth)) 
                {
                    result = (5.35 * cartItem.CartItemUnits);
                }
                // Medium package
                else if ((height <= mPMBHeight) && (width <= mPMBWidth) && (depth <= mPMBDepth)) 
                {
                    result = (11.35 * cartItem.CartItemUnits);
                }
                // Large package
                else if ((height <= lPMBHeight) && (width <= lPMBWidth) && (depth <= lPMBDepth))
                {
                    result = (15.45 * cartItem.CartItemUnits);
                }
                else 
                {
                    // Get shipping cost for chosen shipping option
                    ShippingCostInfo cost = ShippingCostInfoProvider.GetShippingCostInfo(cart.ShoppingCartShippingOptionID, cartItem.UnitWeight);
                    if (cost != null)
                    {
                        result = cost.ShippingCostValue;
                    }
                    else
                    {
                        // No weight-specific cost found -> use general charge
                        result = cart.ShippingOption.ShippingOptionCharge;
                    }
                }
            }
        }

        return result;
    }

    #endregion
}