using System;
using System.Web;
using System.Data;

using CMS.Ecommerce;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.GlobalHelper;

/// <summary>
/// Sample shopping cart info provider. 
/// Can be registered either by replacing the ShoppingCartInfoProvider.ProviderObject (uncomment the line in SampleECommerceModule.cs) or through cms.extensibility section of the web.config
/// </summary>
public class CustomShoppingCartInfoProvider : ShoppingCartInfoProvider
{
    #region "Example: Custom discount applied to the order total price"

    ///// <summary>
    ///// Calculates discount which should be applied to the total items price.
    ///// </summary>
    ///// <param name="cart">Shopping cart</param>        
    //protected override double CalculateOrderDiscountInternal(ShoppingCartInfo cart)
    //{
    //    // Default order discount
    //    double result = base.CalculateOrderDiscountInternal(cart);

    //    // Order total price
    //    double totalPrice = cart.TotalItemsPriceInMainCurrency;

    //    // Example of order discount based on the time of shopping - Happy hours (4 PM - 7 PM)
    //    if ((DateTime.Now.Hour >= 16) && (DateTime.Now.Hour <= 19))
    //    {
    //        // 20% discount 
    //        result = result + totalPrice * 0.2;
    //    }
    //    // Example of order discount based on the total price of all cart items
    //    else if (totalPrice > 500)
    //    {
    //        // 10% discount
    //        result = result + totalPrice * 0.1;
    //    }
    //    // Example of order discount based on the discount coupon
    //    // Please note: You will need to modify ShoppingCartContent.ascx.cs to skip coupon validation for the products
    //    else if ((cart.DiscountCoupon != null) && (cart.DiscountCoupon.IsValid))
    //    {
    //        result = result + ECommerceHelper.GetDiscountValue(totalPrice, cart.DiscountCoupon.DiscountCouponValue, cart.DiscountCoupon.DiscountCouponIsFlatValue);
    //    }

    //    return result;
    //}

    #endregion


    #region "Example: Add product A to the shopping cart -> product B is also added (automaticaly)"

    ///// <summary>
    ///// Adds new item to the shopping cart object and returns its object.
    ///// </summary>
    ///// <param name="cart">Shopping cart</param>
    ///// <param name="itemParams">Parameters from which the new shopping cart item is initialized.</param>
    //protected override ShoppingCartItemInfo AddShoppingCartItemInternal(ShoppingCartInfo cart, ShoppingCartItemParameters itemParams)
    //{
    //    // Add item to the shopping cart
    //    ShoppingCartItemInfo item = base.AddShoppingCartItemInternal(cart, itemParams);

    //    // SKU number of product A
    //    string skuA = "A";

    //    // SKU number of product B
    //    string skuB = "B";

    //    // Add product B to the shopping cart automatically after the product A is added
    //    if (item.SKU.SKUNumber.ToLowerCSafe() == skuA.ToLowerCSafe())
    //    {
    //        string where = string.Format("SKUNumber = N'{0}'", SqlHelperClass.GetSafeQueryString(skuB));
    //        DataSet products = SKUInfoProvider.GetSKUs(where, null);
    //        if (!DataHelper.DataSourceIsEmpty(products))
    //        {
    //            // Get data of the product B
    //            SKUInfo productB = new SKUInfo(products.Tables[0].Rows[0]);

    //            // Configure product B
    //            ShoppingCartItemParameters itemBParams = new ShoppingCartItemParameters(productB.SKUID, itemParams.Quantity);

    //            // Add/update product B in the shopping cart
    //            SetShoppingCartItem(cart, itemBParams);
    //        }
    //    }

    //    return item;
    //}

    #endregion


    #region "Example: Adding custom SKU columns to shopping cart content table and displaying them on the invoice"

    //// These two simple customizations enable you to display values from your custom SKU columns in your order content table on invoice, 
    //// meaning you can then use {%MyColumn1%} and {%MyColumn2%} macros in "Ecommerce.Transformations.Order_ContentTable" TXT/XML transformation 
    //// which is used for displaying invoice content table by default.
    
    ///// <summary>
    ///// Creates an empty shopping cart content table.
    ///// </summary> 
    //protected override DataTable CreateContentTableInternal()
    //{
    //    // Create default table
    //    DataTable table = base.CreateContentTableInternal();

    //    // Add custom culumns
    //    table.Columns.Add(new DataColumn("MyColumn1", typeof(int)));
    //    table.Columns.Add(new DataColumn("MyColumn2", typeof(double)));

    //    return table;
    //}


    ///// <summary>
    ///// Creates one row of the content table with the data which are retrieved from the specified shopping cart item.
    ///// </summary>
    ///// <param name="item">Shopping cart item</param>
    ///// <param name="table">Table from which the content row is created</param>
    //protected override DataRow CreateContentRowInternal(ShoppingCartItemInfo item, DataTable table)
    //{
    //    // Get row with default data
    //    DataRow row = base.CreateContentRowInternal(item, table);

    //    // Set values to custom columns
    //    row["MyColumn1"] = ValidationHelper.GetInteger(item.SKU.GetValue("MyColumn1"), 0);
    //    row["MyColumn2"] = ValidationHelper.GetDouble(item.SKU.GetValue("MyColumn2"), 0);

    //    return row;
    //}

    #endregion


    #region "Webinar samples"

    /// <summary>
    /// Adds new item to the shopping cart object and returns its object.
    /// </summary>
    /// <param name="cart">Shopping cart</param>
    /// <param name="itemParams">Parameters from which the new shopping cart item is initialized.</param>
    protected override ShoppingCartItemInfo AddShoppingCartItemInternal(ShoppingCartInfo cart, ShoppingCartItemParameters itemParams)
    {
        // Add item to the shopping cart
        ShoppingCartItemInfo item = base.AddShoppingCartItemInternal(cart, itemParams);
        if (item != null) 
        {
            // Accepts simple data values
            item.CartItemCustomData["MyCustomValue1"] = "Sample value 1";
            item.CartItemCustomData["MyCustomValue2"] = 25.55;
            item.CartItemCustomData["MyCustomValue3"] = item.SKU.GetValue("MarketPriceSourceName");
            item.CartItemCustomData["MyCustomValue4"] = item.SKU.GetValue("MarketPriceValue");
        }

        return item;
    }

    #endregion
}