using System;

using CMS.Ecommerce;
using CMS.SettingsProvider;
using CMS.DocumentEngine;

/// <summary>
/// Sample e-commerce module class. Partial class ensures correct registration.
/// </summary>
[SampleECommerceModuleLoader]
public partial class CMSModuleLoader
{
    #region "Macro methods loader attribute"

    /// <summary>
    /// Module registration
    /// </summary>
    private class SampleECommerceModuleLoaderAttribute : CMSLoaderAttribute
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public SampleECommerceModuleLoaderAttribute()
        {
            // Require E-commerce module to load properly
            RequiredModules = new string[] { ModuleEntry.ECOMMERCE };
        }


        /// <summary>
        /// Initializes the module
        /// </summary>
        public override void Init()
        {
            #region "Webinar samples"

            // Registering for event fired when lazy-loaded RelatedData property is accessed
            SKUInfo.TYPEINFOSKU.OnLoadRelatedData += new TypeInfo.ObjectLoadRelatedDataEventHandler(TYPEINFOSKU_OnLoadRelatedData);

            // -- Uncomment this line to register the CustomSKUInfoProvider programmatically
            SKUInfoProvider.ProviderObject = new CustomSKUInfoProvider();

            // -- Uncomment this line to register the CustomShoppingCartInfoProvider programmatically
            ShoppingCartInfoProvider.ProviderObject = new CustomShoppingCartInfoProvider();

            // -- Uncomment this line to register the CustomShoppingCartItemInfoProvider programmatically
            ShoppingCartItemInfoProvider.ProviderObject = new CustomShoppingCartItemInfoProvider();            
            
            // -- Uncomment this line to register the CustomShippingOptionInfoProvider programmatically
            ShippingOptionInfoProvider.ProviderObject = new CustomShippingOptionInfoProvider();

            // -- Uncomment this line to register the CustomOrderInfoProvider programmatically
            OrderInfoProvider.ProviderObject = new CustomOrderInfoProvider();

            #endregion

            // This line provides the ability to register the classes via web.config cms.extensibility section from App_Code
            ClassHelper.OnGetCustomClass += GetCustomClass;
        }


        #region "Webinar samples"

        protected object TYPEINFOSKU_OnLoadRelatedData(BaseInfo infoObj)
        {
            CustomDataContainer result = null;

            SKUInfo prod = (infoObj as SKUInfo);
            if (prod != null) 
            {
                result = new CustomDataContainer() { MarketPriceSourceName = "CME Group", MarketPriceValue = 35.47 };
            }

            return result;
        }

        #endregion


        /// <summary>
        /// Gets the custom class object based on the given class name. This handler is called when the assembly name is App_Code.
        /// </summary>
        private static void GetCustomClass(object sender, ClassEventArgs e)
        {
            if (e.Object == null)
            {
                // Provide your custom classes
                switch (e.ClassName)
                {
                        // Define the class CustomShippingOptionInfoProvider inheriting the ShippingOptionInfoProvider and you can customize the provider
                    case "CustomShippingOptionInfoProvider":
                        e.Object = new CustomShippingOptionInfoProvider();
                        break;

                        // Define the class CustomShoppingCartInfoProvider inheriting the ShoppingCartInfoProvider and you can customize the provider
                    case "CustomShoppingCartInfoProvider":
                        e.Object = new CustomShoppingCartInfoProvider();
                        break;

                        // Define the class CustomShoppingCartItemInfoProvider inheriting the ShoppingCartItemInfoProvider and you can customize the provider
                    case "CustomShoppingCartItemInfoProvider":
                        e.Object = new CustomShoppingCartItemInfoProvider();
                        break;

                        // Define the class CustomSKUInfoProvider inheriting the SKUInfoProvider and you can customize the provider
                    case "CustomSKUInfoProvider":
                        e.Object = new CustomSKUInfoProvider();
                        break;

                    // Define the class CustomTaxClassInfoProvider inheriting the TaxClassInfoProvider and you can customize the provider
                    case "CustomTaxClassInfoProvider":
                        e.Object = new CustomTaxClassInfoProvider();
                        break;

                    // Define the class CustomCustomerInfoProvider inheriting the CustomerInfoProvider and you can customize the provider
                    case "CustomCustomerInfoProvider":
                        e.Object = new CustomCustomerInfoProvider();
                        break;

                    // Define the class CustomOrderInfoProvider inheriting the OrderInfoProvider and you can customize the provider
                    case "CustomOrderInfoProvider":
                        e.Object = new CustomOrderInfoProvider();
                        break;
                }
            }
        }
    }

    #endregion
}